@@ -1,8 +1,6 @@
 import inspect
 import re
 import typing
-from collections import OrderedDict
-from decimal import Decimal
 from operator import attrgetter
 
 import uritemplate
@@ -22,14 +20,13 @@ from drf_spectacular.extensions import (
     OpenApiSerializerFieldExtension, OpenApiSerializerExtension
 )
 from drf_spectacular.settings import spectacular_settings
-from drf_spectacular.contrib.authentication import *  # noqa: F403, F401
-from drf_spectacular.contrib.serializers import *  # noqa: F403, F401
+from drf_spectacular.contrib import *  # noqa: F403, F401
 from drf_spectacular.plumbing import (
     build_basic_type, warn, anyisinstance, force_instance, is_serializer,
     follow_field_source, is_field, is_basic_type, build_array_type,
     ComponentRegistry, ResolvedComponent, build_parameter_type, error,
     resolve_regex_path_parameter, safe_ref, has_override, get_override,
-    append_meta, build_object_type,
+    append_meta, build_object_type, build_choice_field,
 )
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import OpenApiParameter
@@ -337,34 +334,6 @@ class AutoSchema(ViewInspector):
 
         return paginator.get_schema_operation_parameters(view)
 
-    def _map_choicefield(self, field):
-        choices = list(OrderedDict.fromkeys(field.choices))  # preserve order and remove duplicates
-        if all(isinstance(choice, bool) for choice in choices):
-            type = 'boolean'
-        elif all(isinstance(choice, int) for choice in choices):
-            type = 'integer'
-        elif all(isinstance(choice, (int, float, Decimal)) for choice in choices):  # `number` includes `integer`
-            # Ref: https://tools.ietf.org/html/draft-wright-json-schema-validation-00#section-5.21
-            type = 'number'
-        elif all(isinstance(choice, str) for choice in choices):
-            type = 'string'
-        else:
-            type = None
-
-        mapping = {
-            # The value of `enum` keyword MUST be an array and SHOULD be unique.
-            # Ref: https://tools.ietf.org/html/draft-wright-json-schema-validation-00#section-5.20
-            'enum': choices
-        }
-
-        # If We figured out `type` then and only then we should set it. It must be a string.
-        # Ref: https://swagger.io/docs/specification/data-models/data-types/#mixed-type
-        # It is optional but it can not be null.
-        # Ref: https://tools.ietf.org/html/draft-wright-json-schema-validation-00#section-5.21
-        if type:
-            mapping['type'] = type
-        return mapping
-
     def _map_model_field(self, model_field, direction):
         assert isinstance(model_field, models.Field)
         # to get a fully initialized serializer field we use DRF's own init logic
@@ -428,6 +397,7 @@ class AutoSchema(ViewInspector):
             schema = self._map_serializer_field(field.child_relation, direction)
             # remove hand-over initkwargs applying only to outer scope
             schema.pop('description', None)
+            schema.pop('readOnly', None)
             return append_meta(build_array_type(schema), meta)
 
         if isinstance(field, serializers.PrimaryKeyRelatedField):
@@ -435,11 +405,16 @@ class AutoSchema(ViewInspector):
             # from parent. also avoid calling Manager. __bool__ as it might be customized
             # to hit the database.
             if getattr(field, 'queryset', None) is not None:
-                schema = self._map_model_field(field.queryset.model._meta.pk, direction)
+                model_field = field.queryset.model._meta.pk
             else:
-                schema = self._map_model_field(field.parent.Meta.model._meta.pk, direction)
+                if isinstance(field.parent, serializers.ManyRelatedField):
+                    model_field = field.parent.parent.Meta.model._meta.pk
+                else:
+                    model_field = field.parent.Meta.model._meta.pk
+
             # primary keys are usually non-editable (readOnly=True) and map_model_field correctly
             # signals that attribute. however this does not apply in the context of relations.
+            schema = self._map_model_field(model_field, direction)
             schema.pop('readOnly', None)
             return append_meta(schema, meta)
 
@@ -455,15 +430,11 @@ class AutoSchema(ViewInspector):
         if isinstance(field, serializers.HyperlinkedRelatedField):
             return append_meta(build_basic_type(OpenApiTypes.URI), meta)
 
-        # ChoiceFields (single and multiple).
-        # Q:
-        # - Is 'type' required?
-        # - can we determine the TYPE of a choicefield?
         if isinstance(field, serializers.MultipleChoiceField):
-            return append_meta(build_array_type(self._map_choicefield(field)), meta)
+            return append_meta(build_array_type(build_choice_field(field.choices)), meta)
 
         if isinstance(field, serializers.ChoiceField):
-            return append_meta(self._map_choicefield(field), meta)
+            return append_meta(build_choice_field(field.choices), meta)
 
         if isinstance(field, serializers.ListField):
             schema = build_array_type({})
@@ -497,6 +468,9 @@ class AutoSchema(ViewInspector):
         if isinstance(field, serializers.UUIDField):
             return append_meta(build_basic_type(OpenApiTypes.UUID), meta)
 
+        if isinstance(field, serializers.DurationField):
+            return append_meta(build_basic_type(OpenApiTypes.STR), meta)
+
         if isinstance(field, serializers.IPAddressField):
             # TODO this might be a DRF bug. protocol is not propagated to serializer although it
             #  should have been. results in always 'both' (thus no format)
@@ -537,8 +511,12 @@ class AutoSchema(ViewInspector):
             return append_meta(content, meta)
 
         if isinstance(field, serializers.FileField):
-            # TODO returns filename. but does it accept binary data on upload?
-            return append_meta(build_basic_type(OpenApiTypes.STR), meta)
+            if spectacular_settings.COMPONENT_SPLIT_REQUEST and direction == 'request':
+                content = build_basic_type(OpenApiTypes.BINARY)
+            else:
+                use_url = getattr(field, 'use_url', api_settings.UPLOADED_FILES_USE_URL)
+                content = build_basic_type(OpenApiTypes.URI if use_url else OpenApiTypes.STR)
+            return append_meta(content, meta)
 
         if isinstance(field, serializers.SerializerMethodField):
             method = getattr(field.parent, field.method_name)
