@@ -1,9 +1,12 @@
+import hashlib
 import inspect
 import json
 import sys
 from abc import ABCMeta
-from collections import defaultdict
-from collections.abc import Hashable
+from collections import defaultdict, OrderedDict
+from collections.abc import Hashable, Iterable
+from decimal import Decimal
+from enum import Enum
 from typing import List, Type, Optional, TypeVar, Union, Generic, DefaultDict
 
 import inflection
@@ -20,6 +23,12 @@ from drf_spectacular.types import (
 )
 from drf_spectacular.utils import OpenApiParameter
 
+try:
+    from django.db.models.enums import Choices  # only available in Django>3
+except ImportError:
+    class Choices:  # type: ignore
+        pass
+
 T = TypeVar('T')
 
 
@@ -194,6 +203,35 @@ def build_parameter_type(
     return schema
 
 
+def build_choice_field(choices):
+    choices = list(OrderedDict.fromkeys(choices))  # preserve order and remove duplicates
+    if all(isinstance(choice, bool) for choice in choices):
+        type = 'boolean'
+    elif all(isinstance(choice, int) for choice in choices):
+        type = 'integer'
+    elif all(isinstance(choice, (int, float, Decimal)) for choice in choices):  # `number` includes `integer`
+        # Ref: https://tools.ietf.org/html/draft-wright-json-schema-validation-00#section-5.21
+        type = 'number'
+    elif all(isinstance(choice, str) for choice in choices):
+        type = 'string'
+    else:
+        type = None
+
+    schema = {
+        # The value of `enum` keyword MUST be an array and SHOULD be unique.
+        # Ref: https://tools.ietf.org/html/draft-wright-json-schema-validation-00#section-5.20
+        'enum': choices
+    }
+
+    # If We figured out `type` then and only then we should set it. It must be a string.
+    # Ref: https://swagger.io/docs/specification/data-models/data-types/#mixed-type
+    # It is optional but it can not be null.
+    # Ref: https://tools.ietf.org/html/draft-wright-json-schema-validation-00#section-5.21
+    if type:
+        schema['type'] = type
+    return schema
+
+
 def build_root_object(paths, components):
     settings = spectacular_settings
     root = {
@@ -256,16 +294,21 @@ def _follow_field_source(model, path):
         # end of traversal
         if isinstance(field_or_property, property):
             return field_or_property.fget
+        elif callable(field_or_property):
+            return field_or_property
         else:
             return get_field_from_model(model, field_or_property)
     else:
-        if isinstance(field_or_property, property):
-            target_model = field_or_property.fget.__annotations__.get('return')
+        if isinstance(field_or_property, property) or callable(field_or_property):
+            if isinstance(field_or_property, property):
+                target_model = field_or_property.fget.__annotations__.get('return')
+            else:
+                target_model = field_or_property.__annotations__.get('return')
             if not target_model:
                 raise UnableToProceedError(
                     f'could not follow field source through intermediate property "{path[0]}" '
-                    f'on model {model}. please add a type hint on the model\'s property to '
-                    f'enable traversal of the source path "{".".join(path)}".'
+                    f'on model {model}. please add a type hint on the model\'s property/function'
+                    f'to enable traversal of the source path "{".".join(path)}".'
                 )
             return _follow_field_source(target_model, path[1:])
         else:
@@ -284,11 +327,11 @@ def follow_field_source(model, path):
         return _follow_field_source(model, path)
     except UnableToProceedError as e:
         warn(e)
-    except:  # noqa: E722
+    except Exception as exc:
         warn(
             f'could not resolve field on model {model} with path "{".".join(path)}". '
             f'this is likely a custom field that does some unknown magic. maybe '
-            f'consider annotating the field/property? defaulting to "string".'
+            f'consider annotating the field/property? defaulting to "string". (Exception: {exc})'
         )
 
     def dummy_property(obj) -> str:
@@ -428,6 +471,51 @@ class OpenApiGeneratorExtension(Generic[T], metaclass=ABCMeta):
         return None
 
 
+def deep_import_string(string):
+    """ augmented import from string, e.g. MODULE.CLASS/OBJECT.ATTRIBUTE """
+    try:
+        return import_string(string)
+    except ImportError:
+        pass
+    try:
+        *path, attr = string.split('.')
+        obj = import_string('.'.join(path))
+        return getattr(obj, attr)
+    except (ImportError, AttributeError):
+        pass
+
+
+def load_enum_name_overrides():
+    overrides = {}
+    for name, choices in spectacular_settings.ENUM_NAME_OVERRIDES.items():
+        if isinstance(choices, str):
+            choices = deep_import_string(choices)
+        if not choices:
+            warn(
+                f'unable to load choice override for {name} from ENUM_NAME_OVERRIDES. '
+                f'please check module path string.'
+            )
+            continue
+        if inspect.isclass(choices) and issubclass(choices, Choices):
+            choices = choices.choices
+        if inspect.isclass(choices) and issubclass(choices, Enum):
+            choices = [c.value for c in choices]
+        if isinstance(choices, Iterable) and isinstance(choices[0], str):
+            choices = [(c, c) for c in choices]
+        overrides[list_hash(list(dict(choices).keys()))] = name
+
+    if len(spectacular_settings.ENUM_NAME_OVERRIDES) != len(overrides):
+        error(
+            'ENUM_NAME_OVERRIDES has duplication issues. encountered multiple names '
+            'for the same choice set. enum naming might be unexpected.'
+        )
+    return overrides
+
+
+def list_hash(lst):
+    return hashlib.blake2b(json.dumps(list(lst), sort_keys=True).encode()).hexdigest()
+
+
 def postprocess_schema_enums(result, generator, **kwargs):
     """
     simple replacement of Enum/Choices that globally share the same name and have
@@ -446,29 +534,56 @@ def postprocess_schema_enums(result, generator, **kwargs):
 
     schemas = result.get('components', {}).get('schemas', {})
 
+    overrides = load_enum_name_overrides()
+
     hash_mapping = defaultdict(set)
-    # collect all enums, their names and contents
+    # collect all enums, their names and choice sets
     for props in iter_prop_containers(list(schemas.values())):
         for prop_name, prop_schema in props.items():
             if 'enum' not in prop_schema:
                 continue
-            hash_mapping[prop_name].add(
-                hash(json.dumps(prop_schema['enum'], sort_keys=True))
-            )
-    # safe replacement requires name to have only one set of enum values
-    candidate_enums = {
-        prop_name for prop_name, prop_hash_set in hash_mapping.items()
-        if len(prop_hash_set) == 1
-    }
-    # replace all valid occurrences with enum schema component
+            hash_mapping[prop_name].add(list_hash(prop_schema['enum']))
+
+    # traverse all enum properties and generate a name for the choice set. naming collisions
+    # are resolved and a warning is emitted. giving a choice set multiple names is technically
+    # correct but potentially unwanted. also emit a warning there to make the user aware.
+    enum_name_mapping = {}
+    for prop_name, prop_hash_set in hash_mapping.items():
+        for prop_hash in prop_hash_set:
+            if prop_hash in overrides:
+                enum_name = overrides[prop_hash]
+            elif len(prop_hash_set) == 1:
+                enum_name = f'{inflection.camelize(prop_name)}Enum'
+            else:
+                enum_name = f'{inflection.camelize(prop_name)}{prop_hash[:3].capitalize()}Enum'
+                warn(
+                    f'automatic enum naming encountered a collision for field "{prop_name}". the '
+                    f'same name has been used for multiple choice sets. the collision was resolved '
+                    f'with {enum_name}. add an entry to ENUM_NAME_OVERRIDES to fix the naming.'
+                )
+            if enum_name_mapping.get(prop_hash, enum_name) != enum_name:
+                warn(
+                    f'encountered multiple names for the same choice set ({enum_name}). this '
+                    f'may be unwanted even though the generated schema is technically correct. '
+                    f'add an entry to ENUM_NAME_OVERRIDES to fix the naming.'
+                )
+                del enum_name_mapping[prop_hash]
+            else:
+                enum_name_mapping[prop_hash] = enum_name
+            enum_name_mapping[(prop_hash, prop_name)] = enum_name
+
+    # replace all enum occurrences with a enum schema component. cut out the
+    # enum, replace it with a reference and add a corresponding component.
     for props in iter_prop_containers(list(schemas.values())):
         for prop_name, prop_schema in props.items():
             if 'enum' not in prop_schema:
                 continue
-            if prop_name not in candidate_enums:
-                continue
 
-            enum_name = f'{inflection.camelize(prop_name)}Enum'
+            prop_hash = list_hash(prop_schema['enum'])
+            # when choice sets are reused under multiple names, the generated name cannot be
+            # resolved from the hash alone. fall back to prop_name and hash for resolution.
+            enum_name = enum_name_mapping.get(prop_hash) or enum_name_mapping[prop_hash, prop_name]
+
             enum_schema = {k: v for k, v in prop_schema.items() if k in ['type', 'enum']}
             prop_schema = {k: v for k, v in prop_schema.items() if k not in ['type', 'enum']}
 
